/*
 Copyright (C) 2012 Almer Lucke
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation 
 files, to deal in FCSSlideMenuContainerController without restriction, including without limitation the rights to use, copy, 
 modify, merge, publish, distribute, sublicense, and/or sell copies of FCSSlideMenuContainerController, and to permit 
 persons to whom FCSSlideMenuContainerController is furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in all copies or substantial portions of 
 FCSSlideMenuContainerController.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE 
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
 COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
 OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


@class FCSSlideMenuController;


@interface FCSSlideMenuContainerController : UIViewController
{
    BOOL _leftMenuOpen;
    BOOL _rightMenuOpen;
    UIPanGestureRecognizer* _panGestureRecognizer;
    CGFloat _menuSnapSpeed;
}

// assign these via nib or programmatically
@property (nonatomic, retain) IBOutlet FCSSlideMenuController* leftMenuViewController;
@property (nonatomic, retain) IBOutlet FCSSlideMenuController* rightMenuViewController;
@property (nonatomic, retain) IBOutlet UIViewController* topContentViewController;

// read only, you can change the layer shadow properties of the contentView for example to
// customize appearance
@property (nonatomic, readonly) UINavigationController* contentNavigationController;
@property (nonatomic, readonly) UIView* leftMenuToggleView;
@property (nonatomic, readonly) UIView* rightMenuToggleView;
@property (nonatomic, readonly) UIView* contentView;
@property (nonatomic, readonly) UIView* leftMenuView;
@property (nonatomic, readonly) UIView* rightMenuView;

// menu slide/snap speed and width of snapping/closing area of menu's
@property (nonatomic, assign) CGFloat menuSnapDuration;
@property (nonatomic, assign) NSInteger menuSnapWidth;

// close the menu
- (void)closeMenu;
- (void)leftMenuToggle;
- (void)rightMenuToggle;

// override by subclasses to set custom navbar button items for left/right menu
- (UIBarButtonItem*)barButtonItemForLeftMenuClosed;
- (UIBarButtonItem*)barButtonItemForLeftMenuOpened;
- (UIBarButtonItem*)barButtonItemForRightMenuClosed;
- (UIBarButtonItem*)barButtonItemForRightMenuOpened;

@end
