/*
 Copyright (C) 2012 Almer Lucke
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation 
 files, to deal in FCSSlideMenuContainerController without restriction, including without limitation the rights to use, copy, 
 modify, merge, publish, distribute, sublicense, and/or sell copies of FCSSlideMenuContainerController, and to permit 
 persons to whom FCSSlideMenuContainerController is furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in all copies or substantial portions of 
 FCSSlideMenuContainerController.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE 
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
 COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
 OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "FCSSlideMenuContainerController.h"
#import "FCSSlideMenuController.h"


@interface FCSSlideMenuContainerController (Private)
- (void)initialize;
- (void)setupNavigationBarButtons;
- (void)setupLeftMenuFrames;
- (void)setupRightMenuFrames;
- (void)panContent:(UIPanGestureRecognizer*)panGestureRecognizer;
- (void)swipeLeftMenuToggle:(UISwipeGestureRecognizer*)swipeGestureRecognizer;
- (void)tapLeftMenuToggle:(UITapGestureRecognizer*)tapGestureRecognizer;
- (void)swipeRightMenuToggle:(UISwipeGestureRecognizer*)swipeGestureRecognizer;
- (void)tapRightMenuToggle:(UITapGestureRecognizer*)tapGestureRecognizer;
@end




@implementation FCSSlideMenuContainerController

#pragma mark - Synthesizers

@synthesize leftMenuViewController = _leftMenuViewController;
@synthesize rightMenuViewController = _rightMenuViewController;
@synthesize topContentViewController = _topContentViewController;
@synthesize contentNavigationController = _contentNavigationController;
@synthesize leftMenuToggleView = _leftMenuToggleView;
@synthesize rightMenuToggleView = _rightMenuToggleView;
@synthesize contentView;
@synthesize leftMenuView = _leftMenuView;
@synthesize rightMenuView = _rightMenuView;
@synthesize menuSnapDuration = _menuSnapDuration;
@synthesize menuSnapWidth = _menuSnapWidth;



#pragma mark - Init/Dealloc


- (void)dealloc
{
    [_leftMenuView release];
    [_rightMenuView release];
    [_leftMenuViewController release];
    [_rightMenuViewController release];
    [_topContentViewController release];
    [_contentNavigationController release];
    [_leftMenuToggleView release];
    [_rightMenuToggleView release];
    
    [super dealloc];
}

- (void)initialize
{
    _menuSnapWidth = 60;
    _menuSnapDuration = 0.35;
}

- (id)init
{
    if((self = [super init]))
    {
        [self initialize];
    }
    
    return self;
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    if((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]))
    {
        [self initialize];
    }
    
    return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
    if((self = [super initWithCoder:aDecoder]))
    {
        [self initialize];
    }
    
    return self; 
}



#pragma mark - View Controller

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    NSInteger menuWidth = self.view.bounds.size.width - _menuSnapWidth;
    _menuSnapSpeed = menuWidth / _menuSnapDuration;
    
    self.contentNavigationController.view.frame = self.view.bounds;
    [self.view addSubview:self.contentNavigationController.view];
    
    if(_topContentViewController)
    {
        self.contentNavigationController.viewControllers = [NSArray arrayWithObject:_topContentViewController];
        _panGestureRecognizer = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(panContent:)];
        [_topContentViewController.view addGestureRecognizer:_panGestureRecognizer];
        [_panGestureRecognizer release];
    }
    
    if(_leftMenuViewController)
    {
        [self.leftMenuView addSubview:_leftMenuViewController.view];
        [self setupLeftMenuFrames];
    }
    
    if(_rightMenuViewController)
    {
        [self.rightMenuView addSubview:_rightMenuViewController.view];
        [self setupRightMenuFrames];
    }
    
    [self setupNavigationBarButtons];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    [_leftMenuViewController viewWillAppear:animated];
    [_rightMenuViewController viewWillAppear:animated];
    [_contentNavigationController viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    [_leftMenuViewController viewDidAppear:animated];
    [_rightMenuViewController viewDidAppear:animated];
    [_contentNavigationController viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
    
    [_leftMenuViewController viewWillDisappear:animated];
    [_rightMenuViewController viewWillDisappear:animated];
    [_contentNavigationController viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
    
    [_leftMenuViewController viewDidDisappear:animated];
    [_rightMenuViewController viewDidDisappear:animated];
    [_contentNavigationController viewDidDisappear:animated];
}

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{
    [super willRotateToInterfaceOrientation:toInterfaceOrientation duration:duration];
    
    [_leftMenuViewController willRotateToInterfaceOrientation:toInterfaceOrientation duration:duration];
    [_rightMenuViewController willRotateToInterfaceOrientation:toInterfaceOrientation duration:duration];
    [_contentNavigationController willRotateToInterfaceOrientation:toInterfaceOrientation duration:duration];
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation
{
    [super didRotateFromInterfaceOrientation:fromInterfaceOrientation];
    
    [_leftMenuViewController didRotateFromInterfaceOrientation:fromInterfaceOrientation];
    [_rightMenuViewController didRotateFromInterfaceOrientation:fromInterfaceOrientation];
    [_contentNavigationController didRotateFromInterfaceOrientation:fromInterfaceOrientation];
}

- (void)willAnimateRotationToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{
    [super willAnimateRotationToInterfaceOrientation:toInterfaceOrientation duration:duration];
    
    [_leftMenuViewController willAnimateRotationToInterfaceOrientation:toInterfaceOrientation duration:duration];
    [_rightMenuViewController willAnimateRotationToInterfaceOrientation:toInterfaceOrientation duration:duration];
    [_contentNavigationController willAnimateRotationToInterfaceOrientation:toInterfaceOrientation duration:duration];
}

- (void) willAnimateFirstHalfOfRotationToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{
    [super willAnimateFirstHalfOfRotationToInterfaceOrientation:toInterfaceOrientation duration:duration];
    
    [_leftMenuViewController willAnimateFirstHalfOfRotationToInterfaceOrientation:toInterfaceOrientation duration:duration];
    [_rightMenuViewController willAnimateFirstHalfOfRotationToInterfaceOrientation:toInterfaceOrientation duration:duration];
    [_contentNavigationController willAnimateFirstHalfOfRotationToInterfaceOrientation:toInterfaceOrientation duration:duration];
}

- (void)didAnimateFirstHalfOfRotationToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation
{
    [super didAnimateFirstHalfOfRotationToInterfaceOrientation:toInterfaceOrientation];
    
    [_leftMenuViewController didAnimateFirstHalfOfRotationToInterfaceOrientation:toInterfaceOrientation];
    [_rightMenuViewController didAnimateFirstHalfOfRotationToInterfaceOrientation:toInterfaceOrientation];
    [_contentNavigationController didAnimateFirstHalfOfRotationToInterfaceOrientation:toInterfaceOrientation];
}

- (void)willAnimateSecondHalfOfRotationFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation duration:(NSTimeInterval)duration
{
    [super willAnimateSecondHalfOfRotationFromInterfaceOrientation:fromInterfaceOrientation duration:duration];
    
    [_leftMenuViewController willAnimateSecondHalfOfRotationFromInterfaceOrientation:fromInterfaceOrientation duration:duration];
    [_rightMenuViewController willAnimateSecondHalfOfRotationFromInterfaceOrientation:fromInterfaceOrientation duration:duration];
    [_contentNavigationController willAnimateSecondHalfOfRotationFromInterfaceOrientation:fromInterfaceOrientation duration:duration];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return UIInterfaceOrientationIsPortrait(interfaceOrientation);
}



#pragma mark - Setup Menu Parts

- (UIView*)leftMenuView
{
    if(!_leftMenuView)
    {
        _leftMenuView = [[UIView alloc] initWithFrame:CGRectZero];
    }
    
    if([self isViewLoaded] && !_leftMenuView.superview)
    {
        [self.view insertSubview:_leftMenuView belowSubview:self.contentView];
    }
    
    return _leftMenuView;
}

- (UIView*)rightMenuView
{
    if(!_rightMenuView)
    {
        _rightMenuView = [[UIView alloc] initWithFrame:CGRectZero];
    }
    
    if([self isViewLoaded] && !_rightMenuView.superview)
    {
        [self.view insertSubview:_rightMenuView belowSubview:self.contentView];
    }
    
    return _rightMenuView;
}

- (UIView*)leftMenuToggleView
{
    if(!_leftMenuToggleView)
    {
        UISwipeGestureRecognizer* leftMenuSwipeGestureRecognizer = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(swipeLeftMenuToggle:)];
        UITapGestureRecognizer* leftMenuTapGestureRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(tapLeftMenuToggle:)];
        
        leftMenuSwipeGestureRecognizer.direction = UISwipeGestureRecognizerDirectionLeft;
        _leftMenuToggleView = [[UIView alloc] initWithFrame:CGRectZero];
        _leftMenuToggleView.backgroundColor = [UIColor clearColor];
        [_leftMenuToggleView addGestureRecognizer:leftMenuSwipeGestureRecognizer];
        [_leftMenuToggleView addGestureRecognizer:leftMenuTapGestureRecognizer];
        [leftMenuSwipeGestureRecognizer release];
        [leftMenuTapGestureRecognizer release];
    }
    
    return _leftMenuToggleView;
}

- (UIView*)rightMenuToggleView
{
    if(!_rightMenuToggleView)
    {
        UISwipeGestureRecognizer* rightMenuSwipeGestureRecognizer = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(swipeRightMenuToggle:)];
        UITapGestureRecognizer* rightMenuTapGestureRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(tapRightMenuToggle:)];
        
        rightMenuSwipeGestureRecognizer.direction = UISwipeGestureRecognizerDirectionRight;
        _rightMenuToggleView = [[UIView alloc] initWithFrame:CGRectZero];
        _rightMenuToggleView.backgroundColor = [UIColor clearColor];
        [_rightMenuToggleView addGestureRecognizer:rightMenuSwipeGestureRecognizer];
        [_rightMenuToggleView addGestureRecognizer:rightMenuTapGestureRecognizer];
        [rightMenuSwipeGestureRecognizer release];
        [rightMenuTapGestureRecognizer release];
    }
    
    return _rightMenuToggleView;
}

- (UINavigationController*)contentNavigationController
{
    if(!_contentNavigationController)
    {
        _contentNavigationController = [[UINavigationController alloc] init];
    }
    
    return _contentNavigationController;
}

- (UIView*)contentView
{
    return self.contentNavigationController.view;
}

- (void)setMenuSnapWidth:(NSInteger)menuSnapWidth
{
    _menuSnapWidth = menuSnapWidth;
    
    if([self isViewLoaded])
    {
        NSInteger menuWidth = self.view.bounds.size.width - _menuSnapWidth;
        _menuSnapSpeed = menuWidth / _menuSnapDuration;
        
        [self setupLeftMenuFrames];
        [self setupRightMenuFrames];
    }
}

- (void)setMenuSnapDuration:(CGFloat)menuSnapDuration
{
    _menuSnapDuration = menuSnapDuration;
    
    if([self isViewLoaded])
    {
        NSInteger menuWidth = self.view.bounds.size.width - _menuSnapWidth;
        _menuSnapSpeed = menuWidth / _menuSnapDuration;
    }
}

- (void)setLeftMenuViewController:(FCSSlideMenuController*)leftMenuViewController
{
    if(_leftMenuViewController)
    {
        [_leftMenuViewController.view removeFromSuperview];
        [_leftMenuViewController release];
        _leftMenuViewController = nil;
    }
    
    if(leftMenuViewController)
    {
        _leftMenuViewController = [leftMenuViewController retain];
        _leftMenuViewController.slideMenuContainer = self;
        
        if([self isViewLoaded])
        {
            [self.leftMenuView addSubview:_leftMenuViewController.view];
            
            [self setupLeftMenuFrames];
        }
    }
    
    if([self isViewLoaded])
    {
        [self setupNavigationBarButtons];
    }
}

- (void)setRightMenuViewController:(FCSSlideMenuController*)rightMenuViewController
{
    if(_rightMenuViewController)
    {
        [_rightMenuViewController.view removeFromSuperview];
        [_rightMenuViewController release];
        _rightMenuViewController = nil;
    }
    
    if(rightMenuViewController)
    {
        _rightMenuViewController = [rightMenuViewController retain];
        _rightMenuViewController.slideMenuContainer = self;
        
        if([self isViewLoaded])
        {
            [self.rightMenuView addSubview:_rightMenuViewController.view];
            
            [self setupRightMenuFrames];
        }
    }
    
    if([self isViewLoaded])
    {
        [self setupNavigationBarButtons];
    }
}

- (void)setTopContentViewController:(UIViewController*)topContentViewController
{
    if(_topContentViewController)
    {
        [_topContentViewController.view removeGestureRecognizer:_panGestureRecognizer];
        [_topContentViewController release];
        _topContentViewController = nil;
    }
    
    _topContentViewController = [topContentViewController retain];
    
    self.contentNavigationController.viewControllers = [NSArray arrayWithObject:_topContentViewController];
    _panGestureRecognizer = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(panContent:)];
    [_topContentViewController.view addGestureRecognizer:_panGestureRecognizer];
    [_panGestureRecognizer release];
    
    [self setupNavigationBarButtons];
}

- (void)setupNavigationBarButtons
{
    if(_topContentViewController)
    {
        if(_leftMenuViewController)
        {
            if(_leftMenuOpen)
            {
                _topContentViewController.navigationItem.leftBarButtonItem = [self barButtonItemForLeftMenuOpened];
            }
            else 
            {
                _topContentViewController.navigationItem.leftBarButtonItem = [self barButtonItemForLeftMenuClosed];
            }
        }
        else 
        {
            _topContentViewController.navigationItem.leftBarButtonItem = nil;
        }
        
        if(_rightMenuViewController)
        {
            if(_rightMenuOpen)
            {
                _topContentViewController.navigationItem.rightBarButtonItem = [self barButtonItemForRightMenuOpened];
            }
            else 
            {
                _topContentViewController.navigationItem.rightBarButtonItem = [self barButtonItemForRightMenuClosed];
            }
        }
        else 
        {
            _topContentViewController.navigationItem.rightBarButtonItem = nil;
        }
    }
}

- (void)setupLeftMenuFrames
{
    NSInteger navBarHeight = self.contentNavigationController.navigationBar.frame.size.height;
    NSInteger menuWidth = self.view.bounds.size.width - _menuSnapWidth;
    NSInteger menuHeight = self.view.bounds.size.height;
    
    CGRect leftMenuFrame = CGRectMake(0, 0, menuWidth, menuHeight);
    CGRect leftMenuToggleViewFrame = CGRectMake(menuWidth, navBarHeight, _menuSnapWidth, menuHeight - navBarHeight);
    
    self.leftMenuView.frame = leftMenuFrame;
    _leftMenuViewController.view.frame = self.leftMenuView.bounds;
    self.leftMenuToggleView.frame = leftMenuToggleViewFrame;
}

- (void)setupRightMenuFrames
{
    NSInteger navBarHeight = self.contentNavigationController.navigationBar.frame.size.height;
    NSInteger menuWidth = self.view.bounds.size.width - _menuSnapWidth;
    NSInteger menuHeight = self.view.bounds.size.height;
    
    CGRect rightMenuFrame = CGRectMake(_menuSnapWidth, 0, menuWidth, menuHeight);
    CGRect rightMenuToggleViewFrame = CGRectMake(0, navBarHeight, _menuSnapWidth, menuHeight - navBarHeight);
    
    self.rightMenuView.frame = rightMenuFrame;
    _rightMenuViewController.view.frame = self.rightMenuView.bounds;
    self.rightMenuToggleView.frame = rightMenuToggleViewFrame;
}



#pragma mark - Override by subclasses To Return BarButtonItems

- (UIBarButtonItem*)barButtonItemForLeftMenuClosed
{
    UIBarButtonItem* item = [[UIBarButtonItem alloc] initWithTitle:@"Left" style:UIBarButtonItemStyleBordered target:self action:@selector(leftMenuToggle)];
    
    return [item autorelease];
}

- (UIBarButtonItem*)barButtonItemForLeftMenuOpened
{
    UIBarButtonItem* item = [[UIBarButtonItem alloc] initWithTitle:@"Left" style:UIBarButtonItemStyleBordered target:self action:@selector(leftMenuToggle)];
    
    return [item autorelease];
}

- (UIBarButtonItem*)barButtonItemForRightMenuOpened
{
    UIBarButtonItem* item = [[UIBarButtonItem alloc] initWithTitle:@"Right" style:UIBarButtonItemStyleBordered target:self action:@selector(rightMenuToggle)];
    
    return [item autorelease];
}

- (UIBarButtonItem*)barButtonItemForRightMenuClosed
{
    UIBarButtonItem* item = [[UIBarButtonItem alloc] initWithTitle:@"Right" style:UIBarButtonItemStyleBordered target:self action:@selector(rightMenuToggle)];
    
    return [item autorelease];
}



#pragma mark - Toggle/Pan/Swipe/Tap

- (void)panContent:(UIPanGestureRecognizer*)panGestureRecognizer
{
    NSInteger menuWidth = self.view.bounds.size.width - _menuSnapWidth;
    CGPoint translation = [panGestureRecognizer translationInView:self.view];
    NSInteger maxTranslationLeft = menuWidth;
    NSInteger maxTranslationRight = menuWidth;
    NSInteger translationX = translation.x;
    CGRect oldFrame = _contentNavigationController.view.frame;
    
    if(translationX < 0)
    {
        if(_rightMenuViewController)
        {
            if(_leftMenuViewController) 
            {
                self.leftMenuView.hidden = YES;
            }
            
            self.rightMenuView.hidden = NO;
            
            CGFloat translationNormalize = abs(translationX) / (CGFloat)maxTranslationLeft;
            CGFloat duration = 0.0;
            
            if(translationNormalize > 1.0) translationNormalize = 1.0;
            
            CGRect newFrame = CGRectMake(translationNormalize * -maxTranslationLeft, oldFrame.origin.y, oldFrame.size.width, oldFrame.size.height);
            
            _contentNavigationController.view.frame = newFrame;
            
            if(panGestureRecognizer.state == UIGestureRecognizerStateEnded)
            {
                CGFloat velocityX = fabs([panGestureRecognizer velocityInView:self.view].x);
                if(velocityX < _menuSnapSpeed) velocityX = _menuSnapSpeed;
                
                if(translationX < -_menuSnapWidth)
                {
                    duration = (menuWidth - translationNormalize * maxTranslationRight) / velocityX;
                    newFrame = CGRectMake(-menuWidth, oldFrame.origin.y, oldFrame.size.width, oldFrame.size.height);
                    _rightMenuOpen = YES;
                    [self.view addSubview:_rightMenuToggleView];
                    [_topContentViewController.navigationItem setRightBarButtonItem:[self barButtonItemForRightMenuOpened] animated:NO];
                    
                }
                else
                {
                    duration = (translationNormalize * maxTranslationLeft) / velocityX;
                    newFrame = CGRectMake(0, oldFrame.origin.y, oldFrame.size.width, oldFrame.size.height);
                }
                
                [UIView animateWithDuration:duration animations:^
                 {
                     _contentNavigationController.view.frame = newFrame;
                 }];
            }
        }
    }
    else if(_leftMenuViewController)
    {
        if(_rightMenuViewController)
        {
            self.rightMenuView.hidden = YES;
        }
        self.leftMenuView.hidden = NO;
        
        CGFloat translationNormalize = translationX / (CGFloat)maxTranslationRight;
        CGFloat duration = 0.0;
        
        if(translationNormalize > 1.0) translationNormalize = 1.0;
        
        CGRect newFrame = CGRectMake(translationNormalize * maxTranslationRight, oldFrame.origin.y, oldFrame.size.width, oldFrame.size.height);
        
        _contentNavigationController.view.frame = newFrame;
        
        if(panGestureRecognizer.state == UIGestureRecognizerStateEnded)
        {
            CGFloat velocityX = fabs([panGestureRecognizer velocityInView:self.view].x);
            if(velocityX < _menuSnapSpeed) velocityX = _menuSnapSpeed;
            
            if(translationX > _menuSnapWidth)
            {
                duration = (menuWidth - translationNormalize * maxTranslationRight) / velocityX;
                newFrame = CGRectMake(menuWidth, oldFrame.origin.y, oldFrame.size.width, oldFrame.size.height);
                _leftMenuOpen = YES;
                [self.view addSubview:_leftMenuToggleView];
                [_topContentViewController.navigationItem setLeftBarButtonItem:[self barButtonItemForLeftMenuOpened] animated:NO];
            }
            else
            {
                duration = (translationNormalize * maxTranslationRight) / velocityX;
                newFrame = CGRectMake(0, oldFrame.origin.y, oldFrame.size.width, oldFrame.size.height);
            }
            
            [UIView animateWithDuration:duration animations:^
             {
                 _contentNavigationController.view.frame = newFrame;
             }];
        }
    }
}

- (void)swipeLeftMenuToggle:(UISwipeGestureRecognizer*)swipeGestureRecognizer
{
    [self leftMenuToggle];
}

- (void)tapLeftMenuToggle:(UITapGestureRecognizer*)tapGestureRecognizer
{
    if(tapGestureRecognizer.state == UIGestureRecognizerStateEnded)
    {
        [self leftMenuToggle];
    }
}

- (void)swipeRightMenuToggle:(UISwipeGestureRecognizer*)swipeGestureRecognizer
{
    [self rightMenuToggle];
}

- (void)tapRightMenuToggle:(UITapGestureRecognizer*)tapGestureRecognizer
{
    if(tapGestureRecognizer.state == UIGestureRecognizerStateEnded)
    {
        [self rightMenuToggle];
    }
}

- (void)leftMenuToggle
{
    NSInteger menuWidth = self.view.bounds.size.width - _menuSnapWidth;
    
    if(_leftMenuViewController)
    {
        CGRect oldFrame = _contentNavigationController.view.frame;
        CGRect newFrame = CGRectZero;
        if(!_leftMenuOpen)
        {
            newFrame = CGRectMake(menuWidth, oldFrame.origin.y, oldFrame.size.width, oldFrame.size.height);
            
            self.leftMenuView.hidden = NO;
            if(_rightMenuViewController)
            {
                self.rightMenuView.hidden = YES;
            }
            [self.view addSubview:_leftMenuToggleView];
            [_topContentViewController.navigationItem setLeftBarButtonItem:[self barButtonItemForLeftMenuOpened] animated:NO];
        }
        else
        {
            newFrame = CGRectMake(0, oldFrame.origin.y, oldFrame.size.width, oldFrame.size.height);
            
            [_leftMenuToggleView removeFromSuperview];
            [_topContentViewController.navigationItem setLeftBarButtonItem:[self barButtonItemForLeftMenuClosed] animated:NO];
        }
        
        [UIView animateWithDuration:self.menuSnapDuration animations:^
         {
             _contentNavigationController.view.frame = newFrame;
         }];
        
        _leftMenuOpen = !_leftMenuOpen;
    }
}

- (void)rightMenuToggle
{
    NSInteger menuWidth = self.view.bounds.size.width - _menuSnapWidth;
    
    if(_rightMenuViewController)
    {
        CGRect oldFrame = _contentNavigationController.view.frame;
        CGRect newFrame = CGRectZero;
        
        if(!_rightMenuOpen)
        {
            newFrame = CGRectMake(-menuWidth, oldFrame.origin.y, oldFrame.size.width, oldFrame.size.height);
            
            if(_leftMenuViewController)
            {
                self.leftMenuView.hidden = YES;
            }
            self.rightMenuView.hidden = NO;
            [self.view addSubview:_rightMenuToggleView];
            [_topContentViewController.navigationItem setRightBarButtonItem:[self barButtonItemForRightMenuOpened] animated:NO];
        }
        else
        {
            newFrame = CGRectMake(0, oldFrame.origin.y, oldFrame.size.width, oldFrame.size.height);
            
            [_rightMenuToggleView removeFromSuperview];
            [_topContentViewController.navigationItem setRightBarButtonItem:[self barButtonItemForRightMenuClosed] animated:NO];
        }
        
        [UIView animateWithDuration:self.menuSnapDuration animations:^
         {
             _contentNavigationController.view.frame = newFrame;
         }];
        
        _rightMenuOpen = !_rightMenuOpen;
    }
}

- (void)closeMenu
{
    if(_leftMenuOpen) [self leftMenuToggle];
    else if(_rightMenuOpen) [self rightMenuToggle];
}

@end
